@@ -23,6 +23,14 @@ use crate::device::physical::QueueFamily;
 use crate::device::Device;
 use crate::device::DeviceOwned;
 use crate::device::Queue;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use crate::memory::pool::alloc_dedicated_with_exportable_fd;
 use crate::memory::pool::AllocFromRequirementsFilter;
 use crate::memory::pool::AllocLayout;
 use crate::memory::pool::MappingRequirement;
@@ -30,19 +38,26 @@ use crate::memory::pool::MemoryPool;
 use crate::memory::pool::MemoryPoolAlloc;
 use crate::memory::pool::PotentialDedicatedAllocation;
 use crate::memory::pool::StdMemoryPoolAlloc;
+use crate::memory::DeviceMemoryAllocError;
 use crate::memory::{DedicatedAlloc, MemoryRequirements};
-use crate::memory::{DeviceMemoryAllocError, ExternalMemoryHandleType};
 use crate::sync::AccessError;
 use crate::sync::Sharing;
 use crate::DeviceSize;
 use smallvec::SmallVec;
-use std::fs::File;
 use std::hash::Hash;
 use std::hash::Hasher;
 use std::marker::PhantomData;
 use std::mem;
 use std::sync::Arc;
 use std::sync::Mutex;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use {crate::memory::ExternalMemoryHandleType, std::fs::File};
 
 /// Buffer whose content is in device-local memory.
 ///
@@ -173,8 +188,14 @@ impl<T: ?Sized> DeviceLocalBuffer<T> {
         }))
     }
 
-    /// Same as `raw` but with exportable fd option for the allocated memory on Linux
-    #[cfg(target_os = "linux")]
+    /// Same as `raw` but with exportable fd option for the allocated memory on Linux/BSD
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub unsafe fn raw_with_exportable_fd<'a, I>(
         device: Arc<Device>,
         size: DeviceSize,
@@ -194,8 +215,8 @@ impl<T: ?Sized> DeviceLocalBuffer<T> {
 
         let (buffer, mem_reqs) = Self::build_buffer(&device, size, usage, &queue_families)?;
 
-        let mem = MemoryPool::alloc_from_requirements_with_exportable_fd(
-            &Device::standard_pool(&device),
+        let mem = alloc_dedicated_with_exportable_fd(
+            device.clone(),
             &mem_reqs,
             AllocLayout::Linear,
             MappingRequirement::DoNotMap,
@@ -208,8 +229,9 @@ impl<T: ?Sized> DeviceLocalBuffer<T> {
                 }
             },
         )?;
-        debug_assert!((mem.offset() % mem_reqs.alignment) == 0);
-        buffer.bind_memory(mem.memory(), mem.offset())?;
+        let mem_offset = mem.offset();
+        debug_assert!((mem_offset % mem_reqs.alignment) == 0);
+        buffer.bind_memory(mem.memory(), mem_offset)?;
 
         Ok(Arc::new(DeviceLocalBuffer {
             inner: buffer,
@@ -245,8 +267,14 @@ impl<T: ?Sized> DeviceLocalBuffer<T> {
 
     /// Exports posix file descriptor for the allocated memory
     /// requires `khr_external_memory_fd` and `khr_external_memory` extensions to be loaded.
-    /// Only works on Linux.
-    #[cfg(target_os = "linux")]
+    /// Only works on Linux/BSD.
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn export_posix_fd(&self) -> Result<File, DeviceMemoryAllocError> {
         self.memory
             .memory()
@@ -280,7 +308,8 @@ unsafe impl<T: ?Sized, A> DeviceOwned for DeviceLocalBuffer<T, A> {
 
 unsafe impl<T: ?Sized, A> BufferAccess for DeviceLocalBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn inner(&self) -> BufferInner {
@@ -369,14 +398,16 @@ where
 
 unsafe impl<T: ?Sized, A> TypedBufferAccess for DeviceLocalBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     type Content = T;
 }
 
 impl<T: ?Sized, A> PartialEq for DeviceLocalBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn eq(&self, other: &Self) -> bool {
@@ -384,11 +415,17 @@ where
     }
 }
 
-impl<T: ?Sized, A> Eq for DeviceLocalBuffer<T, A> where T: 'static + Send + Sync {}
+impl<T: ?Sized, A> Eq for DeviceLocalBuffer<T, A>
+where
+    T: Send + Sync + 'static,
+    A: Send + Sync,
+{
+}
 
 impl<T: ?Sized, A> Hash for DeviceLocalBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
